package com.dayou.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.common.BaseEntity;
import com.dayou.entity.BusinessNumberRecycle;
import com.dayou.entity.Major;
import com.dayou.entity.MajorProduction;
import com.dayou.entity.MajorTarget;
import com.dayou.enums.MainBusinessEnum;
import com.dayou.enums.ProductionEnum;
import com.dayou.enums.TakeNumberEnum;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.BusinessNumberRecycleMapper;
import com.dayou.service.IBusinessNumberRecycleService;
import com.dayou.service.IMajorProductionService;
import com.dayou.service.IMajorService;
import com.dayou.service.IMajorTargetService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static com.dayou.common.Constants.BRI;

@Service
public class BusinessNumberRecycleServiceImpl extends ServiceImpl<BusinessNumberRecycleMapper, BusinessNumberRecycle> implements IBusinessNumberRecycleService {

    @Autowired
    private IMajorTargetService majorTargetService;

    @Autowired
    private IMajorService majorService;

    @Autowired
    private IMajorProductionService majorProductionService;



    @Override
    public Boolean recycleMajorReportNumber(Long businessId) {
        //校验该订单报告号是否可回收
        String mainReportNo = checkOrder(businessId);
        Pattern yearPattern = Pattern.compile("(\\d{4})");
        Pattern numberPattern = Pattern.compile("(?<=字第)\\d+(?=号)");
        Integer year = patternNumber(mainReportNo, yearPattern);
        Integer number = patternNumber(mainReportNo, numberPattern);

        //回收号码
        BusinessNumberRecycle recycle = new BusinessNumberRecycle();
        recycle.setYear(year);
        recycle.setRecycleNo(number);
        recycle.setBusinessType(MainBusinessEnum.MAJOR_BUSINESS.name());
        recycle.setTakeType(TakeNumberEnum.LAND_NUMBER.name());
        return this.save(recycle);
    }

    private String checkOrder(Long businessId) {
        Major major = majorService.getOne(new LambdaQueryWrapper<Major>().eq(BaseEntity::getId, businessId)
                .select(Major::getBusinessObjectType, BaseEntity::getId).eq(BaseEntity::getDeleted,false));
        if (major == null || !TakeNumberEnum.LAND_NUMBER.getType().equals(major.getBusinessObjectType())) {
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"订单类型错误");
        }

        List<MajorTarget> targets = majorTargetService.list(new LambdaQueryWrapper<MajorTarget>().eq(MajorTarget::getMajorId, businessId).eq(MajorTarget::getTargetType, TakeNumberEnum.LAND_NUMBER.getType())
                .eq(BaseEntity::getDeleted, false).select(BaseEntity::getId,MajorTarget::getReportNo));

        if (CollectionUtils.isEmpty(targets)){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"报告号不存在");
        }

        //已取部分子号，其他未取
        List<MajorTarget> notTakeNumberTargets = targets.stream().filter(x -> x.getReportNo() == null).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(notTakeNumberTargets)){
            ErrorCode.throwBusinessException(ErrorCode.IS_RECORD_ERROR);
        }

        //已分开全部取子号
        Set<String> uniReportNo = targets.stream().map(x -> x.getReportNo()).collect(Collectors.toSet());

        if (CollectionUtils.isEmpty(uniReportNo) || uniReportNo.size()!=1){
            ErrorCode.throwBusinessException(ErrorCode.IS_RECORD_ERROR);
        }

        List<MajorProduction> prods = majorProductionService.list(new LambdaQueryWrapper<MajorProduction>().eq(MajorProduction::getMajorId, businessId).eq(MajorProduction::getProduction, ProductionEnum.REPORT.name())
                .eq(BaseEntity::getDeleted, false));

        if (CollectionUtils.isEmpty(prods) || prods.size()!=1){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR);
        }

        String mainReportNo = prods.get(0).getReportNo();

        if (mainReportNo.contains(BRI)){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR);
        }
        return mainReportNo;
    }

    /**
     * 解析数字
     * @param mainReportNo
     * @param pattern
     * @return
     */
    private Integer patternNumber(String mainReportNo,Pattern pattern){
        Matcher matcher = pattern.matcher(mainReportNo);
        if (!matcher.find()){
            ErrorCode.throwBusinessException(ErrorCode.PARAM_ERROR);
        }
        return Integer.valueOf(matcher.group());
    }

}
